package org.richfaces.demo.tables;

import java.io.Serializable;
import java.util.Comparator;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.swing.SortOrder;

import org.richfaces.demo.tables.model.capitals.Capital;

@ManagedBean
@ViewScoped
public class CapitalsSortingBean implements Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = -6237417487105926855L;
    private static final String TIMEZONE_GMT_SEPARATOR = "-";
    private SortOrder capitalsOrder = SortOrder.UNSORTED;
    private SortOrder statesOrder = SortOrder.UNSORTED;
    private SortOrder timeZonesOrder = SortOrder.UNSORTED;

    public void sortByCapitals() {
        statesOrder = SortOrder.UNSORTED;
        timeZonesOrder = SortOrder.UNSORTED;
        if (capitalsOrder.equals(SortOrder.ASCENDING)) {
            setCapitalsOrder(SortOrder.DESCENDING);
        } else {
            setCapitalsOrder(SortOrder.ASCENDING);
        }
    }

    public void sortByStates() {
        capitalsOrder = SortOrder.UNSORTED;
        timeZonesOrder = SortOrder.UNSORTED;
        if (statesOrder.equals(SortOrder.ASCENDING)) {
            setStatesOrder(SortOrder.DESCENDING);
        } else {
            setStatesOrder(SortOrder.ASCENDING);
        }
    }

    public void sortByTimeZones() {
        statesOrder = SortOrder.UNSORTED;
        capitalsOrder = SortOrder.UNSORTED;
        if (timeZonesOrder.equals(SortOrder.ASCENDING)) {
            setTimeZonesOrder(SortOrder.DESCENDING);
        } else {
            setTimeZonesOrder(SortOrder.ASCENDING);
        }
    }

    public Comparator<Capital> getTimeZoneComparator() {
        return new Comparator<Capital>() {

            public int compare(Capital o1, Capital o2) {
                int tz1Int = Integer.valueOf(o1.getTimeZone().split(TIMEZONE_GMT_SEPARATOR)[1]);
                int tz2Int = Integer.valueOf(o2.getTimeZone().split(TIMEZONE_GMT_SEPARATOR)[1]);
                if (tz1Int == tz2Int) {
                    return 0;
                }
                if (tz1Int > tz2Int) {
                    return -1;
                } else {
                    return 1;
                }
            }
        };
    }

    public SortOrder getCapitalsOrder() {
        return capitalsOrder;
    }

    public void setCapitalsOrder(SortOrder capitalsOrder) {
        this.capitalsOrder = capitalsOrder;
    }

    public SortOrder getStatesOrder() {
        return statesOrder;
    }

    public void setStatesOrder(SortOrder statesOrder) {
        this.statesOrder = statesOrder;
    }

    public SortOrder getTimeZonesOrder() {
        return timeZonesOrder;
    }

    public void setTimeZonesOrder(SortOrder timeZonesOrder) {
        this.timeZonesOrder = timeZonesOrder;
    }

}
